@@ -219,21 +219,21 @@ void AudioFile::create(const char *name, Info *myinfo, bool exclusive, timeout_t
 		info.order = 0;
 		info.format = wave;
 	}
-	else if(!stricmp(ext, ".ul"))
+	else if(!stricmp(ext, ".ul") || !stricmp(ext, ".ulaw") || !stricmp(ext, ".mulaw"))
 	{
 		info.encoding = mulawAudio;
 		info.format = raw;
 		info.order = 0;
 		info.rate = 8000;
 	}
-	else if(!stricmp(ext, ".al"))
+	else if(!stricmp(ext, ".al") || !stricmp(ext, ".alaw"))
 	{
 		info.encoding = alawAudio;
 		info.format = raw;
 		info.order = 0;
 		info.rate = 8000;
 	}
-	else if(!stricmp(ext, ".sw"))
+	else if(!stricmp(ext, ".sw") || !stricmp(ext, ".pcm"))
 	{
 		info.encoding = pcm16Mono;
 		info.format = raw;
@@ -246,6 +246,16 @@ void AudioFile::create(const char *name, Info *myinfo, bool exclusive, timeout_t
 		info.order = 0;
 		info.rate = 6000;
 	}
+	else if(!stricmp(ext, ".gsm"))
+	{
+		info.encoding = gsmVoice;
+		info.format = raw;
+		info.order = 0;
+		info.rate = 8000;
+		info.framecount = 160;
+		info.framesize = 33;
+		info.bitrate = 13200;
+	}
 	else if(!stricmp(ext, ".adpcm"))
 	{
 		info.encoding = g721ADPCM;
@@ -260,6 +270,13 @@ void AudioFile::create(const char *name, Info *myinfo, bool exclusive, timeout_t
 		info.order = 0;
 		info.rate = 8000;
 	}
+	else if(!stricmp(ext, ".sx"))
+	{
+		info.encoding = sx96Voice;
+		info.format = raw;
+		info.order = 0;
+		info.rate = 8000;
+	}
 	else if(!stricmp(ext, ".a40"))
 	{
 		info.encoding = g723_5bit;
@@ -687,19 +704,19 @@ void AudioFile::open(const char *name, Mode m, timeout_t framing)
 	if(!ext)
 		goto done;
 
-	if(!stricmp(ext, ".ul"))
+	if(!stricmp(ext, ".ul") || !stricmp(ext, ".ulaw") || !stricmp(ext, ".mulaw"))
 	{
 		info.encoding = mulawAudio;
 		goto done;
 	}
 
-	if(!stricmp(ext, ".al"))
+	if(!stricmp(ext, ".al") || !stricmp(ext, ".alaw"))
 	{
 		info.encoding = alawAudio;
 		goto done;
 	}
 
-	if(!stricmp(ext, ".sw") || !stricmp(ext, ".raw"))
+	if(!stricmp(ext, ".sw") || !stricmp(ext, ".raw") || !stricmp(ext, ".pcm"))
 	{
 		info.encoding = pcm16Mono;
 		goto done;
@@ -886,6 +903,15 @@ void Audio::Info::setFraming(timeout_t timeout)
 	if(!timeout)
 		return;
 
+	if(framing)
+	{
+		timeout = (timeout / framing);
+		if(!timeout)
+			timeout = framing;
+		else
+			timeout = timeout * framing;
+	}
+
 	switch(timeout)
 	{
 	case 10:
@@ -1027,6 +1053,9 @@ unsigned AudioFile::getLinear(Linear addr, unsigned samples)
 	int count;
 	AudioCodec *codec;
 
+	if(!samples)
+		samples = info.framecount;
+
 	if(getEncoding() == pcm16Mono)
 	{
 		count = getBuffer(addr, samples * 2);
@@ -1062,6 +1091,9 @@ unsigned AudioFile::putLinear(Linear addr, unsigned samples)
 	int count;
 	AudioCodec *codec;
 
+	if(!samples)
+		samples = info.framecount;
+
 	if(getEncoding() == pcm16Mono)
 	{
 		count = putBuffer(addr, samples * 2);
@@ -1102,6 +1134,9 @@ int AudioFile::getBuffer(void *addr, unsigned bytes)
 	int count;
 	unsigned long curpos, xfer = 0;
 
+	if(!bytes)
+		bytes = info.framesize;
+
     curpos = (unsigned long)toBytes(info, getPosition());
 	if(curpos >= iolimit && mode == modeFeed)
 	{
@@ -1166,6 +1201,10 @@ Audio::Error AudioFile::getSamples(void *addr, unsigned request)
 	char *fname;
 	unsigned char *caddr = (unsigned char *)addr;
 	int count, bytes;
+
+	if(!request)
+		request = info.framecount;
+
 	for(;;)
 	{
 		bytes = (int)toBytes(info, request);
@@ -1223,6 +1262,9 @@ int AudioFile::putBuffer(void *addr, unsigned len)
 	int count;
 	unsigned long curpos;
 
+	if(!len)
+		len = info.framesize;
+
 	curpos = (unsigned long)toBytes(info, getPosition());
         if(curpos >= iolimit && mode == modeFeed)
         {
@@ -1252,7 +1294,12 @@ int AudioFile::putBuffer(void *addr, unsigned len)
 Audio::Error AudioFile::putSamples(void *addr, unsigned samples)
 {
 	int count;
-	int bytes = (int)toBytes(info, samples);
+	int bytes;
+
+	if(!samples)
+		samples = info.framecount;
+
+	bytes = (int)toBytes(info, samples);
 	if(bytes < 1)
 		return setError(errRequestInvalid);
 
@@ -1323,5 +1370,21 @@ bool AudioFile::isSigned(void)
 	}
 }
 
+Audio::Error AudioFile::position(const char *timestamp)
+{
+	timeout_t pos = toTimeout(timestamp);
 
+	pos /= info.framing;
+	return setPosition(pos * info.framecount);
+}
+
+void AudioFile::getPosition(char *timestamp, size_t size)
+{
+	timeout_t pos = getAbsolutePosition();
+
+	pos /= info.framecount;
+	toTimestamp(pos * info.framing, timestamp, size);
+}
+	
+	
 
