@@ -38,9 +38,10 @@
 #include "private.h"
 #include "audio2.h"
 #include <cmath>
+#include <cctype>
+#include <cstdio>
 
 #ifndef	W32
-#include <cstdio>
 #include <cstdlib>
 #include <unistd.h>
 #include <sys/stat.h>
@@ -145,6 +146,41 @@ float Audio::todbm(Level l)
 	return (float)(log10(sqrt((float)l)/(32768.0*0.70711))*20.0 + M_PI);
 }
 
+const char *Audio::getExtension(Encoding encoding)
+{
+	switch(encoding)
+	{
+	case cdaStereo:
+	case pcm16Stereo:
+	case pcm32Stereo:
+	case cdaMono:
+	case pcm16Mono:
+	case pcm32Mono:
+		return ".wav";
+	case alawAudio:
+		return ".al";
+	case gsmVoice:
+		return ".gsm";
+	case msgsmVoice:
+		return ".wav";
+	case mp1Audio:
+		return ".mp1";
+	case mp2Audio:
+		return ".mp2";
+	case mp3Audio:
+		return ".mp3";
+	case voxADPCM:
+		return ".vox";
+	case sx73Voice:
+	case sx96Voice:
+		return ".sx";
+	case g721ADPCM:
+		return ".adpcm";
+	default:
+		return ".au";
+	}
+}
+	
 Audio::Encoding Audio::getMono(Encoding encoding)
 {
 	switch(encoding)
@@ -222,6 +258,24 @@ Audio::Encoding Audio::getEncoding(const char *name)
 		return g723_3bit;
 	else if(!stricmp(name, "g723_40") || !stricmp(name, "g.723_40"))
 		return g723_5bit;
+	else if(!stricmp(name, ".al"))
+		return alawAudio;
+	else if(!stricmp(name, ".ul"))
+		return mulawAudio;
+	else if(!stricmp(name, ".adpcm"))
+		return g721ADPCM;
+	else if(!stricmp(name, ".cda"))
+		return cdaStereo;
+	else if(!stricmp(name, ".sx"))
+		return sx96Voice;
+	else if(!stricmp(name, ".gsm"))
+		return gsmVoice;
+	else if(!stricmp(name, ".mp1"))
+		return mp1Audio;
+	else if(!stricmp(name, ".mp2"))
+		return mp2Audio;
+	else if(!stricmp(name, ".mp3"))
+		return mp3Audio;
 	else
 		return unknownEncoding;
 }
@@ -758,7 +812,6 @@ Audio::timeout_t Audio::getFraming(Encoding encoding, timeout_t timeout)
 	case gsmVoice:
 		fa = 20;
 		break;
-		return 20;
 	case sx96Voice:
 	case sx73Voice:
 		fa = 15;
@@ -958,6 +1011,65 @@ bool Audio::isStereo(Encoding encoding)
 	return !isMono(encoding);
 }
 
+#ifdef	WIN32
+#define	snprintf _snprintf
+#endif
+
+void Audio::toTimestamp(timeout_t duration, char *buf, size_t len)
+{
+	timeout_t msec = duration % 1000;
+	timeout_t secs = (duration / 1000) % 60;
+	timeout_t mins = (duration / 60000) % 60;
+	timeout_t hours = (duration /3600000);
+
+	snprintf(buf, len, "%ld:%02ld:%02ld.%03ld", hours, mins, secs, msec);
+}
+
+Audio::timeout_t Audio::toTimeout(const char *buf)
+{
+	const char *cp, *sp;
+	timeout_t msec = 0;
+	timeout_t sec = 0;
+
+	cp = strchr(buf, '.');
+	if(cp)
+	{
+		msec = atol(cp + 1);
+		sp = --cp;
+	}
+	else
+		sp = strrchr(buf, ':');
+	
+	if(!sp)
+	{
+		sp = buf;
+		while(*sp && isdigit(*sp))
+			++sp;
+		if(*sp && tolower(*sp) == 'm' && tolower(sp[1] == 's'))
+			return atol(buf);
+		if(tolower(*sp) == 'h')
+			return atol(buf) * 3600000;
+		if(tolower(*sp) == 'm')
+			return atol(buf) * 60000;
+		return atol(buf) * 1000l;
+	}
+	while(*sp != ':' && sp > buf)
+		--sp;
+
+	if(sp == buf)
+		return atol(buf) * 1000 + msec;
+	
+	sec = atol(sp + 1) * 1000;
+	--sp;
+	while(*sp != ':' && sp > buf)
+		--sp;
+	
+	if(sp == buf)
+		return atol(buf) * 60000 + sec + msec;
+	
+	return atol(buf) * 3600000 + atol(++sp) * 60000 + sec + msec;
+}	
+
 bool Audio::loadPlugin(const char *filename)
 {
 #if defined(HAVE_MACH_DYLD)
