@@ -100,6 +100,9 @@ AudioStream::AudioStream() : AudioFile()
 	codec = NULL;
 	framebuf = NULL;
 	bufferFrame = NULL;
+	encBuffer = NULL;
+	decBuffer = NULL;
+	encSize = decSize = 0;
 	bufferPosition = 0;
 }
 
@@ -172,6 +175,14 @@ void AudioStream::close(void)
 	if(framebuf)
 		delete[] framebuf;
 
+	if(encBuffer)
+		delete[] encBuffer;
+
+	if(decBuffer)
+		delete[] decBuffer;
+
+	encSize = decSize = 0;
+	encBuffer = decBuffer = NULL;
 	framebuf = NULL;
 	codec = NULL;
 	AudioFile::close();
@@ -519,3 +530,123 @@ unsigned AudioStream::bufAudio(Linear samples, unsigned count, unsigned size)
 	return frames;
 }
 
+unsigned AudioStream::getEncoded(Encoded addr, unsigned frames)
+{
+	unsigned count = 0, len;
+
+	if(isLinear(info.encoding))
+		return getMono((Linear)addr, frames);
+
+	while(frames--)
+	{
+		len = getBuffer(addr, info.framesize);
+		if(len < info.framesize)
+			break;
+		addr += info.framesize;
+		++count;
+	}
+	return count;
+}			
+
+unsigned AudioStream::putEncoded(Encoded addr, unsigned frames)
+{
+	unsigned count = 0, len;
+
+	if(isLinear(info.encoding))
+		return putMono((Linear)addr, frames);
+
+	while(frames--)
+	{
+		len = putBuffer(addr, info.framesize);
+		if(len < info.framesize)
+			break;
+		addr += info.framesize;
+		++count;
+	}
+	return count;
+}			
+
+unsigned AudioStream::getEncoded(AudioCodec *codec, Encoded addr, unsigned frames)
+{
+	Info ci;
+	unsigned count = 0;
+	unsigned bufsize = 0;
+	unsigned subcount, used = 0;
+	bool eof = false;
+
+	if(!codec)
+		return getEncoded(addr, frames);
+
+	ci = codec->getInfo();
+
+	if(ci.encoding == info.encoding && ci.framecount == info.framecount)
+		return getEncoded(addr, frames);
+
+	if(!isStreamable())
+		return 0;
+
+	while(bufsize < ci.framesize)
+		bufsize += info.framesize;
+
+	if(encSize != bufsize)
+	{
+		if(encBuffer)
+			delete[] encBuffer;
+
+		encBuffer = new Sample[bufsize];
+		encSize = bufsize;
+	}
+
+	while(count < frames && !eof)
+	{	
+		while(used < ci.framesize)
+		{
+			if(getMono(encBuffer + used, 1) < 1)
+			{
+				eof = true;
+				break;
+			}
+			used += info.framesize;
+		}
+		codec->encode(encBuffer, addr, ci.framesize);
+		if(ci.framesize < used)
+			memcpy(encBuffer, encBuffer + ci.framesize, used - ci.framesize);
+		used -= ci.framesize;
+	}
+	return count;
+}
+
+unsigned AudioStream::putEncoded(AudioCodec *codec, Encoded addr, unsigned frames)
+{
+	Info ci;
+	unsigned count = 0;
+
+        if(!codec)
+                return putEncoded(addr, frames);
+
+        ci = codec->getInfo();
+        if(ci.encoding == info.encoding && ci.framecount == info.framecount)
+                return putEncoded(addr, frames);
+
+	if(!isStreamable())
+		return 0;
+
+	if(ci.framecount != decSize)
+	{
+		if(decBuffer)
+			delete[] decBuffer;
+		decBuffer = new Sample[ci.framecount];
+		decSize = ci.framecount;
+	}
+	
+	while(count < frames)
+	{
+		codec->decode(decBuffer, addr, ci.framecount);
+		if(bufMono(decBuffer, ci.framecount) < ci.framecount)
+			break;
+		++count;
+		addr += ci.framesize;
+	}
+
+        return count;
+}
